def get_timestamp(data, arg, single_row=False, multiple_row=False, row_section=False):
    """
    Returns the timestamp of the csv file made from the lidar recording. Based on what boolean the user set to be true,
    the timestamp will output for those row(s).

    Parameters:

        data: Pandas dataframe
            A pandas dataframe used to extract values.
        arg: int
            The row(s) number to be read.
        single_row: boolean
            If this is set to true, output the timestamps for the row that the user selected.
        multiple_row: boolean
            If this is set to true, output the timestamps for the rows that the user selected.
        row_section: boolean
            If this is set to true, output the timestamps for the row sections the user selected.

    Return:

        An array of all the timestamps the user desired to read.
    """

    timestamp = []

    print("NOTE: All timestamps measured in seconds and each element is a timestamp for single azimuth block")
    if single_row is True:
        for col in range(1, len(data.columns)-1, 788):
            timestamp_segment = list(data.values[arg, col:col+8])
            timestamp_segment.reverse()
            timestamp_segment = [int(elem) for index, elem in enumerate(timestamp_segment)]
            del timestamp_segment[0:3]
            timestamp_segment[0] = float(timestamp_segment[0])
            time = float("".join(map(str, timestamp_segment)))
            timestamp.append(time)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            timestamp.append("new row")
            for col in range(1, len(data.columns)-1, 788):
                timestamp_segment = list(data.values[elem, col:col+8])
                timestamp_segment.reverse()
                timestamp_segment = [int(elem) for index, elem in enumerate(timestamp_segment)]
                del timestamp_segment[0:3]
                timestamp_segment[0] = float(timestamp_segment[0])
                time = float("".join(map(str, timestamp_segment)))
                timestamp.append(time)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            timestamp.append("new row")
            for col in range(1, len(data.columns)-1, 788):
                timestamp_segment = list(data.values[row, col:col+8])
                timestamp_segment.reverse()
                timestamp_segment = [int(elem) for index, elem in enumerate(timestamp_segment)]
                del timestamp_segment[0:3]
                timestamp_segment[0] = float(timestamp_segment[0])
                time = float("".join(map(str, timestamp_segment)))
                timestamp.append(time)

    return timestamp


def get_frame_id(data, arg, single_row=False, multiple_row=False, row_section=False):
    """
    Returns the frame ID of the csv file made from the lidar recording. Based on what boolean the user set to be true,
    the frame ID will output for those row(s).

    Parameters:

        data: Pandas dataframe
            A pandas dataframe used to extract values.
        arg: int
            The row(s) number to be read.
        single_row: boolean
            If this is set to true, output the frame IDs for the row that the user selected.
        multiple_row: boolean
            If this is set to true, output the frame IDs for the rows that the user selected.
        row_section: boolean
            If this is set to true, output the frame IDs for the row sections the user selected.

    Return:

        An array of all the frame IDs the user desired to read.
    """
    frame_id = []

    print("NOTE: Each element is a frame ID for single azimuth block")
    if single_row is True:
        for col in range(9, len(data.columns) - 1, 788):
            frame_id_segment = list(data.values[arg, col:col + 2])
            frame_id_segment.reverse()
            frame_id_segment = [int(elem) for index, elem in enumerate(frame_id_segment)]
            frame = int("".join(map(str, frame_id_segment)))
            frame_id.append(frame)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            frame_id.append("new row")
            for col in range(9, len(data.columns) - 1, 788):
                frame_id_segment = list(data.values[elem, col:col + 2])
                frame_id_segment.reverse()
                frame_id_segment = [int(elem) for index, elem in enumerate(frame_id_segment)]
                frame = int("".join(map(str, frame_id_segment)))
                frame_id.append(frame)
    elif row_section is True:
        for row in range(arg[0], arg[1] + 1):
            frame_id.append("new row")
            for col in range(9, len(data.columns) - 1, 788):
                frame_id_segment = list(data.values[row, col:col + 2])
                frame_id_segment.reverse()
                frame_id_segment = [int(elem) for index, elem in enumerate(frame_id_segment)]
                frame = int("".join(map(str, frame_id_segment)))
                frame_id.append(frame)

    return frame_id


def get_measurement_id(data, arg, single_row=False, multiple_row=False, row_section=False):
    """
    Returns the measurement ID of the csv file made from the lidar recording. Based on what boolean the user set to be true,
    the measurement ID will output for those row(s).

    Parameters:

        data: Pandas dataframe
            A pandas dataframe used to extract values.
        arg: int
            The row(s) number to be read.
        single_row: boolean
            If this is set to true, output the measurement IDs for the row that the user selected.
        multiple_row: boolean
            If this is set to true, output the measurement IDs for the rows that the user selected.
        row_section: boolean
            If this is set to true, output the measurement IDs for the row sections the user selected.

    Return:

        An array of all the measurement IDs the user desired to read.
    """
    measurement_id = []

    print("NOTE: Each element is a measurement ID for single azimuth block")
    if single_row is True:
        for col in range(11, len(data.columns)-1,788):
            measurement_id_segment = list(data.values[arg, col:col+2])
            measurement_id_segment.reverse()
            measurement_id_segment = [int(elem) for index, elem in enumerate(measurement_id_segment)]
            measurement = int("".join(map(str, measurement_id_segment)))
            measurement_id.append(measurement)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            measurement_id.append("new row")
            for col in range(11, len(data.columns)-1,788):
                measurement_id_segment = list(data.values[elem, col:col+2])
                measurement_id_segment.reverse()
                measurement_id_segment = [int(elem) for index, elem in enumerate(measurement_id_segment)]
                measurement = int("".join(map(str, measurement_id_segment)))
                measurement_id.append(measurement)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            measurement_id.append("new row")
            for col in range(11, len(data.columns)-1, 788):
                measurement_id_segment = list(data.values[row, col:col+2])
                measurement_id_segment.reverse()
                measurement_id_segment = [int(elem) for index, elem in enumerate(measurement_id_segment)]
                measurement = int("".join(map(str, measurement_id_segment)))
                measurement_id.append(measurement)

    return measurement_id


def get_encoder_count(data, arg, single_row=False, multiple_row=False, row_section=False):
    """
    Returns the encoder count of the csv file made from the lidar recording. Based on what boolean the user set to be true,
    the encoder count will output for those row(s).

    Parameters:

        data: Pandas dataframe
            A pandas dataframe used to extract values.
        arg: int
            The row(s) number to be read.
        single_row: boolean
            If this is set to true, output the encoder count for the row that the user selected.
        multiple_row: boolean
            If this is set to true, output the encoder count for the rows that the user selected.
        row_section: boolean
            If this is set to true, output the encoder count for the row sections the user selected.

    Return:

        An array of all the encoder count the user desired to read.
    """

    encoder_count = []

    print("NOTE: Each element is a encoder count for single azimuth block")
    if single_row is True:
        for col in range(13, len(data.columns) - 1, 788):
            encoder_count_segment = list(data.values[arg, col:col + 4])
            encoder_count_segment.reverse()
            encoder_count_segment = [int(elem) for index, elem in enumerate(encoder_count_segment)]
            encoder = int("".join(map(str, encoder_count_segment)))
            encoder_count.append(encoder)

    elif multiple_row is True:
        for index, elem in enumerate(arg):
            encoder_count.append('new row')
            for col in range(13, len(data.columns) - 1, 788):
                encoder_count_segment = list(data.values[elem, col:col + 4])
                encoder_count_segment.reverse()
                encoder_count_segment = [int(elem) for index, elem in enumerate(encoder_count_segment)]
                encoder = int("".join(map(str, encoder_count_segment)))
                encoder_count.append(encoder)
    elif row_section is True:
        for row in range(arg[0], arg[1] + 1):
            encoder_count.append('new row')
            for col in range(13, len(data.columns) - 1, 788):
                encoder_count_segment = list(data.values[row, col:col + 4])
                encoder_count_segment.reverse()
                encoder_count_segment = [int(elem) for index, elem in enumerate(encoder_count_segment)]
                encoder = int("".join(map(str, encoder_count_segment)))
                encoder_count.append(encoder)

    return encoder_count


def get_signal_photons(data, arg, single_row=False, multiple_row=False, row_section=False, azimuth_block=0):
    """
    Returns the signal photons of the csv file made from the lidar recording. Based on what boolean the user set to be true,
    the signal photons will output for those row(s).

    Parameters:

        azimuth_block: int
            Specifies which azimuth to read signal photons values from.
        single_row: boolean
            If this is set to true, output the signal photons for the row that the user selected.
        multiple_row: boolean
            If this is set to true, output the signal photons for the rows that the user selected.
        row_section: boolean
            If this is set to true, output the signal photons for the row sections the user selected.

    Return:

        An array of all the signal photons the user desired to read.
    """
    signal_photons = []

    print("NOTE: Each element is a signal photon count for single channel in azimuth block")
    if single_row is True:
        for col in (range(22+azimuth_block, azimuth_block+784, 12)):
            signal_photons_segment = list(data.values[arg, col:col+2])
            signal_photons_segment = [int(elem) for index, elem in enumerate(signal_photons_segment)]
            signl_p = int("".join(map(str, signal_photons_segment)))
            signal_photons.append(signl_p)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            signal_photons.append('new row')
            for col in (range(22+azimuth_block, azimuth_block+784, 12)):
                signal_photons_segment = list(data.values[elem, col:col+2])
                signal_photons_segment = [int(elem) for index, elem in enumerate(signal_photons_segment)]
                signl_p = int("".join(map(str, signal_photons_segment)))
                signal_photons.append(signl_p)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            signal_photons.append('new row')
            for col in (range(22+azimuth_block, azimuth_block+784, 12)):
                signal_photons_segment = list(data.values[row, col:col+2])
                signal_photons_segment = [int(elem) for index, elem in enumerate(signal_photons_segment)]
                signl_p = int("".join(map(str, signal_photons_segment)))
                signal_photons.append(signl_p)

    return signal_photons


def get_range(data, arg, single_row=False, multiple_row=False, row_section=False, azimuth_block=0):
    """
    Returns the range of the csv file made from the lidar recording. Based on what boolean the user set to be true,
    the range will output for those row(s).

    Parameters:

        data: Pandas dataframe
            A pandas dataframe used to extract values.
        arg: int
            The row(s) number to be read.
        azimuth_block: int
            Specifies which azimuth to read range values from.
        single_row: boolean
            If this is set to true, output the range for the row that the user selected.
        multiple_row: boolean
            If this is set to true, output the range for the rows that the user selected.
        row_section: boolean
            If this is set to true, output the erange for the row sections the user selected.

    Return:

        An array of all the range the user desired to read.
    """
    range_mm = []

    print("NOTE: Each element is a range in millimeters for single channel in azimuth block")
    if single_row is True:
        for col in (range(16+azimuth_block, azimuth_block+784, 12)):
            range_segment = list(data.values[arg, col:col+3])
            range_segment = [int(elem) for index, elem in enumerate(range_segment)]
            rang = int("".join(map(str, range_segment)))
            range_mm.append(rang)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            range_mm.append('new row')
            for col in (range(16+azimuth_block, azimuth_block+784, 12)):
                range_segment = list(data.values[elem, col:col+3])
                range_segment = [int(elem) for index, elem in enumerate(range_segment)]
                rang = int("".join(map(str, range_segment)))
                range_mm.append(rang)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            range_mm.append('new row')
            for col in (range(16+azimuth_block, azimuth_block+784, 12)):
                range_segment = list(data.values[row, col:col+3])
                range_segment = [int(elem) for index, elem in enumerate(range_segment)]
                rang = int("".join(map(str, range_segment)))
                range_mm.append(rang)

    return range_mm


def get_reflectivity(data, arg, single_row=False, multiple_row=False, row_section=False, azimuth_block=0):
    """
    Returns the reflectivity of the csv file made from the lidar recording. Based on what boolean the user set to be true,
    the reflectivity will output for those row(s).

    Parameters:

        data: Pandas dataframe
            A pandas dataframe used to extract values.
        arg: int
            The row(s) number to be read.
        azimuth_block: int
            Specifies which azimuth to read reflectivity values from.
        single_row: boolean
            If this is set to true, output the reflectivity for the row that the user selected.
        multiple_row: boolean
            If this is set to true, output the reflectivity for the rows that the user selected.
        row_section: boolean
            If this is set to true, output the reflectivity for the row sections the user selected.

    Return:

        An array of all the reflectivity the user desired to read.
    """
    reflectivity = []

    print("NOTE: Each element is a reflectivity for single channel in azimuth block")
    if single_row is True:
        for col in (range(20+azimuth_block, azimuth_block+784, 12)):
            reflectivity_segment = list(data.values[arg, col:col+2])
            reflectivity_segment = [int(elem) for index, elem in enumerate(reflectivity_segment)]
            reflect = int("".join(map(str, reflectivity_segment)))
            reflectivity.append(reflect)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            reflectivity.append('new row')
            for col in (range(20+azimuth_block, azimuth_block+784, 12)):
                reflectivity_segment = list(data.values[elem, col:col+2])
                reflectivity_segment = [int(elem) for index, elem in enumerate(reflectivity_segment)]
                reflect = int("".join(map(str, reflectivity_segment)))
                reflectivity.append(reflect)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            reflectivity.append('new row')
            for col in (range(20+azimuth_block, azimuth_block+784, 12)):
                reflectivity_segment = list(data.values[row, col:col+2])
                reflectivity_segment = [int(elem) for index, elem in enumerate(reflectivity_segment)]
                reflect = int("".join(map(str, reflectivity_segment)))
                reflectivity.append(reflect)

    return reflectivity


def get_noise_photons(data, arg, single_row=False, multiple_row=False, row_section=False, azimuth_block=0):
    """
    Returns the noise photons of the csv file made from the lidar recording. Based on what boolean the user set to be true,
    the noise photons will output for those row(s).

    Parameters:

        data: Pandas dataframe
            A pandas dataframe used to extract values.
        arg: int
            The row(s) number to be read.
        azimuth_block: int
            Specifies which azimuth to read noise photons values from.
        single_row: boolean
            If this is set to true, output the noise photons for the row that the user selected.
        multiple_row: boolean
            If this is set to true, output the noise photons for the rows that the user selected.
        row_section: boolean
            If this is set to true, output the noise photons for the row sections the user selected.

    Return:

        An array of all the noise photons the user desired to read.
    """
    noise_photons = []

    print("NOTE: Each element is a noise photon count for single channel in azimuth block")
    if single_row is True:
        for col in (range(24+azimuth_block, azimuth_block+788, 12)):
            noise_photons_segment = list(data.values[arg, col:col+3])
            noise_photons_segment = [int(elem) for index, elem in enumerate(noise_photons_segment)]
            noise_p = int("".join(map(str, noise_photons_segment)))
            noise_photons.append(noise_p)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            noise_photons.append('new row')
            for col in (range(24+azimuth_block, azimuth_block+788, 12)):
                noise_photons_segment = list(data.values[elem, col:col+3])
                noise_photons_segment = [int(elem) for index, elem in enumerate(noise_photons_segment)]
                noise_p = int("".join(map(str, noise_photons_segment)))
                noise_photons.append(noise_p)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            noise_photons.append('new row')
            for col in (range(24+azimuth_block, azimuth_block+788, 12)):
                noise_photons_segment = list(data.values[row, col:col+3])
                noise_photons_segment = [int(elem) for index, elem in enumerate(noise_photons_segment)]
                noise_p = int("".join(map(str, noise_photons_segment)))
                noise_photons.append(noise_p)


    return noise_photons



def get_IMU_time(data, arg, single_row=False, multiple_row=False, row_section=False):
    """
    Takes in a file and row arguement(s) along with a boolean that specifies the amount of rows to read the IMU timestamps for.

    Parameter:

        data: file
            The csv file that is passed in to read.
        arg: int
            The row(s) that are to be read.
        single_row: boolean
            If chosen the IMU timestamp for a single row specified by arg will be output.
        multiple_row: boolean
            If chosen the IMU timestamp for multiple rows specified by arg will be output.
        row_section: boolean
            If chosen the IMU timestamp for a row section by arg will be output.

    Return:

        An array of IMU timestamps.
    """
    IMU_time = []
    
    print("NOTE: All timestamps measured in seconds")
    if single_row is True:
        IMU_time_segment = list(data.values[arg, 1:9])
        IMU_time_segment.reverse()
        IMU_time_segment = [int(elem) for index, elem in enumerate(IMU_time_segment)]
        del IMU_time_segment[0:3]
        IMU_time_segment[0] = float(IMU_time_segment[0])
        IMU = float("".join(map(str, IMU_time_segment)))
        IMU_time.append(IMU)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            IMU_time.append('new row')
            IMU_time_segment = list(data.values[elem, 1:9])
            IMU_time_segment.reverse()
            IMU_time_segment = [int(elem) for index, elem in enumerate(IMU_time_segment)]
            del IMU_time_segment[0:3]
            IMU_time_segment[0] = float(IMU_time_segment[0])
            IMU = float("".join(map(str, IMU_time_segment)))
            IMU_time.append(IMU)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            IMU_time.append('new row')
            IMU_time_segment = list(data.values[row, 1:9])
            IMU_time_segment.reverse()
            IMU_time_segment = [int(elem) for index, elem in enumerate(IMU_time_segment)]
            del IMU_time_segment[0:3]
            IMU_time_segment[0] = float(IMU_time_segment[0])
            IMU = float("".join(map(str, IMU_time_segment)))
            IMU_time.append(IMU)
            
    return IMU_time


def get_accel_time(data, arg, single_row=False, multiple_row=False, row_section=False):
    """
    Takes in a file and row arguement(s) along with a boolean that specifies the amount of rows to read the accelerometer timestamps for.

    Parameter:

        data: file
            The csv file that is passed in to read.
        arg: int
            The row(s) that are to be read.
        single_row: boolean
            If chosen the accelerometer timestamp for a single row specified by arg will be output.
        multiple_row: boolean
            If chosen the accelerometer timestamp for multiple rows specified by arg will be output.
        row_section: boolean
            If chosen the accelerometer timestamp for a row section by arg will be output.

    Return:

        An array of accelerometer timestamps.
    """
    accel_time = []
    
    if single_row is True:
        accel_time_segment = list(data.values[arg, 9:17])
        accel_time_segment.reverse()
        accel_time_segment = [int(elem) for index, elem in enumerate(accel_time_segment)]
        del accel_time_segment[0:3]
        accel_time_segment[0] = float(accel_time_segment[0])
        accel = float("".join(map(str, accel_time_segment)))
        accel_time.append(accel)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            accel_time.append('new row')
            accel_time_segment = list(data.values[elem, 9:17])
            accel_time_segment.reverse()
            accel_time_segment = [int(elem) for index, elem in enumerate(accel_time_segment)]
            del accel_time_segment[0:3]
            accel_time_segment[0] = float(accel_time_segment[0])
            accel = float("".join(map(str, accel_time_segment)))
            accel_time.append(accel)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            accel_time.append('new row')
            accel_time_segment = list(data.values[row, 9:17])
            accel_time_segment.reverse()
            accel_time_segment = [int(elem) for index, elem in enumerate(accel_time_segment)]
            del accel_time_segment[0:3]
            accel_time_segment[0] = float(accel_time_segment[0])
            accel = float("".join(map(str, accel_time_segment)))
            accel_time.append(accel)
            
    return accel_time


def get_gyro_time(data, arg, single_row=False, multiple_row=False, row_section=False):
    """
    Takes in a file and row arguement(s) along with a boolean that specifies the amount of rows to read the gyroscope timestamps for.

    Parameter:

        data: file
            The csv file that is passed in to read.
        arg: int
            The row(s) that are to be read.
        single_row: boolean
            If chosen the gyroscope timestamp for a single row specified by arg will be output.
        multiple_row: boolean
            If chosen the gyroscope timestamp for multiple rows specified by arg will be output.
        row_section: boolean
            If chosen the gyroscope timestamp for a row section by arg will be output.

    Return:

        An array of gyroscope timestamps.
    """
    gyro_time = []
    
    if single_row is True:
        gyro_time_segment = list(data.values[arg, 17:25])
        gyro_time_segment.reverse()
        gyro_time_segment = [int(elem) for index, elem in enumerate(gyro_time_segment)]
        del gyro_time_segment[0:3]
        gyro_time_segment[0] = float(gyro_time_segment[0])
        gyro = float("".join(map(str, gyro_time_segment)))
        gyro_time.append(gyro)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            gyro_time.append('new row')
            gyro_time_segment = list(data.values[elem, 17:25])
            gyro_time_segment.reverse()
            gyro_time_segment = [int(elem) for index, elem in enumerate(gyro_time_segment)]
            del gyro_time_segment[0:3]
            gyro_time_segment[0] = float(gyro_time_segment[0])
            gyro = float("".join(map(str, gyro_time_segment)))
            gyro_time.append(gyro)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            gyro_time.append('new row')
            gyro_time_segment = list(data.values[row, 17:25])
            gyro_time_segment.reverse()
            gyro_time_segment = [int(elem) for index, elem in enumerate(gyro_time_segment)]
            del gyro_time_segment[0:3]
            gyro_time_segment[0] = float(gyro_time_segment[0])
            gyro = float("".join(map(str, gyro_time_segment)))
            gyro_time.append(gyro)
            
    return gyro_time


def get_x_accel(data, arg, single_row=False, multiple_row=False, row_section=False):
    """
    Takes in a file and row arguement(s) along with a boolean that specifies the amount of rows to read the x acceleration for.

    Parameter:

        data: file
            The csv file that is passed in to read.
        arg: int
            The row(s) that are to be read.
        single_row: boolean
            If chosen the x acceleration for a single row specified by arg will be output.
        multiple_row: boolean
            If chosen the x acceleration for multiple rows specified by arg will be output.
        row_section: boolean
            If chosen the x acceleration for a row section by arg will be output.

    Return:

        An array of x acceleration.
    """
    x_accel = []
    
    if single_row is True:
        x_accel_segment = list(data.values[arg, 25:29])
        x_accel_segment.reverse()
        x = int("".join(map(str, x_accel_segment)))
        x_accel.append(x)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            x_accel.append('new row')
            x_accel_segment = list(data.values[elem, 25:29])
            x_accel_segment.reverse()
            x = int("".join(map(str, x_accel_segment)))
            x_accel.append(x)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            x_accel.append('new row')
            x_accel_segment = list(data.values[row, 25:29])
            x_accel_segment.reverse()
            x = int("".join(map(str, x_accel_segment)))
            x_accel.append(x)
            
    return x_accel


def get_y_accel(data, arg, single_row=False, multiple_row=False, row_section=False):
    """
    Takes in a file and row arguement(s) along with a boolean that specifies the amount of rows to read the y acceleration for.

    Parameter:

        data: file
            The csv file that is passed in to read.
        arg: int
            The row(s) that are to be read.
        single_row: boolean
            If chosen the y acceleration for a single row specified by arg will be output.
        multiple_row: boolean
            If chosen the y acceleration for multiple rows specified by arg will be output.
        row_section: boolean
            If chosen the y acceleration for a row section by arg will be output.

    Return:

        An array of y acceleration.
    """
    y_accel = []
    
    if single_row is True:
        y_accel_segment = list(data.values[arg, 29:33])
        y_accel_segment.reverse()
        y = int("".join(map(str, y_accel_segment)))
        y_accel.append(y)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            y_accel.append('new row')
            y_accel_segment = list(data.values[elem, 29:33])
            y_accel_segment.reverse()
            y = int("".join(map(str, y_accel_segment)))
            y_accel.append(y)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            y_accel.append('new row')
            y_accel_segment = list(data.values[row, 29:33])
            y_accel_segment.reverse()
            y = int("".join(map(str, y_accel_segment)))
            y_accel.append(y)
            
    return y_accel


def get_z_accel(data, arg, single_row=False, multiple_row=False, row_section=False):
    """
    Takes in a file and row arguement(s) along with a boolean that specifies the amount of rows to read the z acceleration for.

    Parameter:

        data: file
            The csv file that is passed in to read.
        arg: int
            The row(s) that are to be read.
        single_row: boolean
            If chosen the z acceleration for a single row specified by arg will be output.
        multiple_row: boolean
            If chosen the z acceleration for multiple rows specified by arg will be output.
        row_section: boolean
            If chosen the z acceleration for a row section by arg will be output.

    Return:

        An array of z acceleration.
    """
    z_accel = []
    
    if single_row is True:
        z_accel_segment = list(data.values[arg, 33:37])
        z_accel_segment.reverse()
        z = int("".join(map(str, z_accel_segment)))
        z_accel.append(z)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            z_accel.append('new row')
            z_accel_segment = list(data.values[elem, 33:37])
            z_accel_segment.reverse()
            z = int("".join(map(str, z_accel_segment)))
            z_accel.append(z)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            z_accel.append('new row')
            z_accel_segment = list(data.values[row, 33:37])
            z_accel_segment.reverse()
            z = int("".join(map(str, z_accel_segment)))
            z_accel.append(z)
            
    return z_accel


def get_x_ang_vel(data, arg, single_row=False, multiple_row=False, row_section=False):
    """
    Takes in a file and row arguement(s) along with a boolean that specifies the amount of rows to read the x angular velocity for.

    Parameter:

        data: file
            The csv file that is passed in to read.
        arg: int
            The row(s) that are to be read.
        single_row: boolean
            If chosen the x angular velocity for a single row specified by arg will be output.
        multiple_row: boolean
            If chosen the x angular velocity for multiple rows specified by arg will be output.
        row_section: boolean
            If chosen the x angular velocity for a row section by arg will be output.

    Return:

        An array of x angular velocity.
    """
    x_ang_vel = []
    
    if single_row is True:
        x_ang_vel_segment = list(data.values[arg, 37:41])
        x_ang_vel_segment.reverse()
        x_ang = int("".join(map(str, x_ang_vel_segment)))
        x_ang_vel.append(x_ang)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            x_ang_vel.append('new row')
            x_ang_vel_segment = list(data.values[elem, 37:41])
            x_ang_vel_segment.reverse()
            x_ang = int("".join(map(str, x_ang_vel_segment)))
            x_ang_vel.append(x_ang)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            x_ang_vel.append('new row')
            x_ang_vel_segment = list(data.values[row, 37:41])
            x_ang_vel_segment.reverse()
            x_ang = int("".join(map(str, x_ang_vel_segment)))
            x_ang_vel.append(x_ang)
            
    return x_ang_vel


def get_y_ang_vel(data, arg, single_row=False, multiple_row=False, row_section=False):
    """
    Takes in a file and row arguement(s) along with a boolean that specifies the amount of rows to read the y angular velocity for.

    Parameter:

        data: file
            The csv file that is passed in to read.
        arg: int
            The row(s) that are to be read.
        single_row: boolean
            If chosen the y angular velocity for a single row specified by arg will be output.
        multiple_row: boolean
            If chosen the y angular velocity for multiple rows specified by arg will be output.
        row_section: boolean
            If chosen the y angular velocity for a row section by arg will be output.

    Return:

        An array of y angular velocity.
    """
    y_ang_vel = []
    
    if single_row is True:
        y_ang_vel_segment = list(data.values[arg, 41:45])
        y_ang_vel_segment.reverse()
        y_ang = int("".join(map(str, y_ang_vel_segment)))
        y_ang_vel.append(y_ang)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            y_ang_vel.append('new row')
            y_ang_vel_segment = list(data.values[elem, 41:45])
            y_ang_vel_segment.reverse()
            y_ang = int("".join(map(str, y_ang_vel_segment)))
            y_ang_vel.append(y_ang)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            y_ang_vel.append('new row')
            y_ang_vel_segment = list(data.values[row, 41:45])
            y_ang_vel_segment.reverse()
            y_ang = int("".join(map(str, y_ang_vel_segment)))
            y_ang_vel.append(y_ang)
            
    return y_ang_vel


def get_z_ang_vel(data, arg, single_row=False, multiple_row=False, row_section=False):
    """
    Takes in a file and row arguement(s) along with a boolean that specifies the amount of rows to read the z angular velocity for.

    Parameter:

        data: file
            The csv file that is passed in to read.
        arg: int
            The row(s) that are to be read.
        single_row: boolean
            If chosen the z angular velocity for a single row specified by arg will be output.
        multiple_row: boolean
            If chosen the z angular velocity for multiple rows specified by arg will be output.
        row_section: boolean
            If chosen the z angular velocity for a row section by arg will be output.

    Return:

        An array of z angular velocity.
    """
    z_ang_vel = []
    
    if single_row is True:
        z_ang_vel_segment = list(data.values[arg, 45:49])
        z_ang_vel_segment.reverse()
        z_ang = int("".join(map(str, z_ang_vel_segment)))
        z_ang_vel.append(z_ang)
    elif multiple_row is True:
        for index, elem in enumerate(arg):
            z_ang_vel.append('new row')
            z_ang_vel_segment = list(data.values[elem, 45:49])
            z_ang_vel_segment.reverse()
            z_ang = int("".join(map(str, z_ang_vel_segment)))
            z_ang_vel.append(z_ang)
    elif row_section is True:
        for row in range(arg[0], arg[1]+1):
            z_ang_vel.append('new row')
            z_ang_vel_segment = list(data.values[row, 45:49])
            z_ang_vel_segment.reverse()
            z_ang = int("".join(map(str, z_ang_vel_segment)))
            z_ang_vel.append(z_ang)
            
    return z_ang_vel



from tkinter import *
from tkinter import filedialog as fd
from lidar_extract import *
from TSW_IWR import *
from X4_threshold import *
from X4_parser import *
from imu_extract import *
from xyz_calc import *
from PIL import ImageTk, Image
import pandas

window = Tk()
window.title('Lidar Data Extraction Tool')

menubar = Menu(window)
file_menu = Menu(menubar, tearoff=0)
lidar_extract_menu = Menu(menubar, tearoff=0)
IMU_extract_menu = Menu(menubar, tearoff=0)
xyz_cartesian = Menu(menubar, tearoff=0)
X4 = Menu(menubar, tearoff=0)
TSW = Menu(menubar, tearoff=0)

menubar.add_cascade(label="File", menu=file_menu)
menubar.add_cascade(label="Lidar Extract", menu=lidar_extract_menu)
menubar.add_cascade(label="IMU Extract", menu=IMU_extract_menu)
menubar.add_cascade(label="Lidar XYZ Calulator", menu=xyz_cartesian)
menubar.add_cascade(label="Novelda X4", menu=X4)
menubar.add_cascade(label="TSW IWR-1642", menu=TSW)


label_font = ('times', 20, 'bold')
Label(window, text="Welcome to CUDRDC lidar data extraction tool", font=label_font).place(x=50, y=25)

# Loading in images from directory
carleton_img = ImageTk.PhotoImage(Image.open("carleton.png"))
drdc_img = ImageTk.PhotoImage(Image.open("drdc.png"))
carleton_panel = Label(window, image=carleton_img)
drdc_panel = Label(window, image=drdc_img)

# Placing images
drdc_panel.place(x=400, y=75)
carleton_panel.place(x=200, y=75)


def print_list(lst):
    """
    Takes in an array and prints it to textbox.

    Parameters:

        lst: list
            A list object of the data that will be read to user.
    """
    t.delete('1.0', END)
    t.place(x=75, y=250, height=300, width=200)
    for x in lst:
        t.insert(END, str(x) + '\n')


def command(entry):
    """
    Takes in a number of rows to be read from file and maps to list of ints.

    Paramters:

        entry: int
            The row numbers that will be output.

    Return:

        mapped list of data
    """
    w = (entry.get().split(","))
    return list(map(int, w))


def openfile():
    """
    Opens the csv file for reading lidar and IMU packet parameters.
    """
    filename = fd.askopenfilename(filetypes=(("csv file", "*.csv"), ("All files", "*.*")))
    f = open(filename)
    f.read()
    global data
    data = pandas.read_csv(filename)


def lidar_single_row():
    """
    GUI window for reading lidar data in a single row

    Return:

        A textbox of the parameter data user wanted to read.
    """
    newwin = Toplevel(window)
    newwin.geometry("500x600")

    global t
    t = Text(newwin)
    t.place(x=75, y=250, height=300, width=200)

    row_num = IntVar(newwin)
    row_choice = ['Choose Row']
    for i in range(0, data.shape[0]):
        row_choice.append(i)

    row_num.set('Choose Row')
    popupMenu = OptionMenu(newwin, row_num, *row_choice).place(x=0, y=0, width=150)

    azimuth_block_num = IntVar(newwin)
    azimuth_choices = ['Choose Azimuth Block', 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15]
    azimuth_block_num.set('Choose Azimuth Block')
    popupMenu = OptionMenu(newwin, azimuth_block_num, *azimuth_choices).place(x=175, y=0, width=200)

    Label(newwin, text="Datablock parameter:").place(x=175, y=75)

    timestamp = Button(newwin, text='Get Timestamp', fg="red", command=lambda: print_list(get_timestamp(data, row_num.get(), single_row=True))).place(x=0, y=100, width=150)
    frame_id = Button(newwin, text='Get Frame Id', fg="red", command=lambda: print_list(get_frame_id(data, row_num.get(), single_row=True))).place(x=0, y=125, width=150)
    measurement_id = Button(newwin, text='Get Measurement Id', fg="red", command=lambda: print_list(get_measurement_id(data, row_num.get(), single_row=True))).place(x=0, y=150, width=150)
    encoder_count = Button(newwin, text='Get Encoder Count', fg="red", command=lambda: print_list(get_encoder_count(data, row_num.get(), single_row=True))).place(x=0, y=175, width=150)
    signal_photon = Button(newwin, text='Get Signal Photons', fg="red", command=lambda: print_list(get_signal_photons(data, row_num.get(), single_row=True, azimuth_block=azimuth_block_num.get()))).place(x=175, y=100, width=150)
    range_mm = Button(newwin, text='Get Range', fg="red", command=lambda: print_list(get_range(data, row_num.get(), single_row=True, azimuth_block=azimuth_block_num.get()))).place(x=175, y=125, width=150)
    reflectivity = Button(newwin, text='Get Reflectivity', fg="red", command=lambda: print_list(get_reflectivity(data, row_num.get(), single_row=True, azimuth_block=azimuth_block_num.get()))).place(x=175, y=150, width=150)
    noise_photon = Button(newwin, text='Get Noise Photons', fg="red", command=lambda: print_list(get_noise_photons(data, row_num.get(), single_row=True, azimuth_block=azimuth_block_num.get()))).place(x=175, y=175, width=150)


def lidar_multiple_row():
    """
    GUI window for reading lidar data of multiple rows

    Return:

        A textbox of the parameter data user wanted to read.
    """
    newwin = Toplevel(window)
    newwin.geometry("500x600")

    global t
    t = Text(newwin)
    t.place(x=75, y=250, height=300, width=200)

    Label(newwin, text="Enter Rows").place(x=0,y=0)
    entry1 = Entry(newwin)
    entry1.place(x=0,y=25,width=50)

    Label(newwin, text="Azimuth Block").place(x=100, y=0)
    azimuth_block_num = IntVar(newwin)
    azimuth_choices = ['Choose Azimuth Block', 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15]
    azimuth_block_num.set(0)
    popupMenu = OptionMenu(newwin, azimuth_block_num, *azimuth_choices).place(x=100,y=25, width=50)

    Label(newwin, text="Datablock parameter:").place(x=175, y=75)

    timestamp = Button(newwin, text='Get Timestamp', fg="red", command=lambda:print_list(get_timestamp(data, command(entry1), multiple_row=True))).place(x=0, y=100, width=150)
    frame_id = Button(newwin, text='Get Frame Id', fg="red", command=lambda: print_list(get_frame_id(data, command(entry1), multiple_row=True))).place(x=0, y=125, width=150)
    measurement_id = Button(newwin, text='Get Measurement Id', fg="red", command=lambda: print_list(get_measurement_id(data, command(entry1), multiple_row=True))).place(x=0, y=150, width=150)
    encoder_count = Button(newwin, text='Get Encoder Count', fg="red", command=lambda: print_list(get_encoder_count(data, command(entry1), multiple_row=True))).place(x=0, y=175, width=150)
    signal_photon = Button(newwin, text='Get Signal Photons', fg="red", command=lambda: print_list(get_signal_photons(data, command(entry1), multiple_row=True, azimuth_block=azimuth_block_num.get()))).place(x=175, y=100, width=150)
    range_mm = Button(newwin, text='Get Range', fg="red", command=lambda: print_list(get_range(data, command(entry1), multiple_row=True, azimuth_block=azimuth_block_num.get()))).place(x=175, y=125, width=150)
    reflectivity = Button(newwin, text='Get Reflectivity', fg="red", command=lambda: print_list(get_signal_photons(data, command(entry1), multiple_row=True, azimuth_block=azimuth_block_num.get()))).place(x=175, y=150, width=150)
    noise_photon = Button(newwin, text='Get Noise Photons', fg="red", command=lambda: print_list(get_signal_photons(data, command(entry1), multiple_row=True, azimuth_block=azimuth_block_num.get()))).place(x=175, y=175, width=150)


def lidar_row_section():
    """
    GUI window for reading lidar data of the row section

    Return:

        A textbox of the parameter data user wanted to read.
    """
    newwin = Toplevel(window)
    newwin.geometry("500x600")

    global t
    t = Text(newwin)
    t.place(x=75, y=250, height=300, width=200)

    Label(newwin, text="Enter rows").place(x=0, y=0)
    entry1 = Entry(newwin)
    entry1.place(x=0, y=25, width=50)

    Label(newwin, text="Azimuth Block").place(x=100, y=0)
    azimuth_block_num = IntVar(newwin)
    azimuth_choices = ['Choose Azimuth Block', 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15]
    azimuth_block_num.set(0)
    popupMenu = OptionMenu(newwin, azimuth_block_num, *azimuth_choices).place(x=100, y=25, width=50)

    Label(newwin, text="Datablock parameter:").place(x=175, y=75)

    timestamp = Button(newwin, text='Get Timestamp', fg="red", command=lambda: print_list(get_timestamp(data, command(entry1), row_section=True))).place(x=0, y=100, width=150)
    frame_id = Button(newwin, text='Get Frame Id', fg="red", command=lambda: print_list(get_frame_id(data, command(entry1), row_section=True))).place(x=0, y=125, width=150)
    measurement_id = Button(newwin, text='Get Measurement Id', fg="red", command=lambda: print_list(get_measurement_id(data, command(entry1), row_section=True))).place(x=0, y=150, width=150)
    encoder_count = Button(newwin, text='Get Encoder Count', fg="red", command=lambda: print_list(get_encoder_count(data, command(entry1), row_section=True))).place(x=0, y=175, width=150)
    signal_photon = Button(newwin, text='Get Signal Photons', fg="red", command=lambda: print_list(get_signal_photons(data, command(entry1), row_section=True, azimuth_block=azimuth_block_num.get()))).place(x=175, y=100, width=150)
    range_mm = Button(newwin, text='Get Range', fg="red", command=lambda: print_list(get_range(data, command(entry1), row_section=True, azimuth_block=azimuth_block_num.get()))).place(x=175, y=125, width=150)
    reflectivity = Button(newwin, text='Get Reflectivity', fg="red", command=lambda: print_list(get_signal_photons(data, command(entry1), row_section=True, azimuth_block=azimuth_block_num.get()))).place(x=175, y=150, width=150)
    noise_photon = Button(newwin, text='Get Noise Photons', fg="red", command=lambda: print_list(get_signal_photons(data, command(entry1), row_section=True, azimuth_block=azimuth_block_num.get()))).place(x=175, y=175, width=150)


def imu_single_row():
    """
    GUI window for reading IMU data of the single row

    Return:

        A textbox of the parameter data user wanted to read.
    """
    newwin = Toplevel(window)
    newwin.geometry("500x600")

    global t
    t = Text(newwin)
    t.place(x=75, y=250, height=300, width=200)

    row_num = IntVar(newwin)
    row_choice = ['Choose Row']
    for i in range(0, data.shape[0]):
        row_choice.append(i)

    row_num.set('Choose Row')
    popupMenu = OptionMenu(newwin, row_num, *row_choice).place(x=0, y=0, width=150)

    imu_time = Button(newwin, text="Get IMU Timestamp", fg="red", command=lambda: print_list(get_IMU_time(data, row_num.get(), single_row=True))).place(x=0, y=50, width=150)
    accel_time = Button(newwin, text="Get Accelerometer Timestamp", fg="red", command=lambda: print_list(get_accel_time(data, row_num.get(), single_row=True))).place(x=0, y=75, width=150)
    gyro_time = Button(newwin, text="Get Gyroscope Timestamp", fg="red", command=lambda: print_list(get_gyro_time(data, row_num.get(), single_row=True))).place(x=0, y=100, width=150)
    x_accel = Button(newwin, text="Get x acceleration", fg="red", command=lambda: print_list(get_x_accel(data, row_num.get(), single_row=True))).place(x=0, y=125, width=150)
    y_accel = Button(newwin, text="Get y acceleration", fg="red", command=lambda: print_list(get_y_accel(data, row_num.get(), single_row=True))).place(x=175, y=50, width=150)
    z_accel = Button(newwin, text="Get z acceleration", fg="red", command=lambda: print_list(get_z_accel(data, row_num.get(), single_row=True))).place(x=175, y=75, width=150)
    x_ang_vel = Button(newwin, text="Get x angular velocity", fg="red", command=lambda: print_list(get_x_ang_vel(data, row_num.get(), single_row=True))).place(x=175, y=100, width=150)
    y_ang_vel = Button(newwin, text="Get y angular velocity", fg="red", command=lambda: print_list(get_y_ang_vel(data, row_num.get(), single_row=True))).place(x=175, y=125, width=150)
    z_ang_vel = Button(newwin, text="Get z angular velocity", fg="red", command=lambda: print_list(get_z_ang_vel(data, row_num.get(), single_row=True))).place(x=175, y=150, width=150)


def imu_multiple_row():
    """
    GUI window for reading IMU data of multiple rows

    Return:

        A textbox of the parameter data user wanted to read.
    """
    newwin = Toplevel(window)
    newwin.geometry("500x600")

    global t
    t = Text(newwin)
    t.place(x=75, y=250, height=300, width=200)

    Label(newwin, text="Enter Rows").place(x=0, y=0)
    entry1 = Entry(newwin)
    entry1.place(x=0, y=25, width=50)

    imu_time = Button(newwin, text="Get IMU Timestamp", fg="red", command=lambda: print_list(get_IMU_time(data, command(entry1), multiple_row=True))).place(x=0, y=50, width=150)
    accel_time = Button(newwin, text="Get Accelerometer Timestamp", fg="red", command=lambda: print_list(get_accel_time(data, command(entry1), multiple_row=True))).place(x=0, y=75, width=150)
    gyro_time = Button(newwin, text="Get Gyroscope Timestamp", fg="red", command=lambda: print_list(get_gyro_time(data, command(entry1), multiple_row=True))).place(x=0, y=100, width=150)
    x_accel = Button(newwin, text="Get x acceleration", fg="red", command=lambda: print_list(get_x_accel(data, command(entry1), multiple_row=True))).place(x=0, y=125, width=150)
    y_accel = Button(newwin, text="Get y acceleration", fg="red", command=lambda: print_list(get_y_accel(data, command(entry1), multiple_row=True))).place(x=175, y=50, width=150)
    z_accel = Button(newwin, text="Get z acceleration", fg="red", command=lambda: print_list(get_z_accel(data, command(entry1), multiple_row=True))).place(x=175, y=75, width=150)
    x_ang_vel = Button(newwin, text="Get x angular velocity", fg="red", command=lambda: print_list(get_x_ang_vel(data, command(entry1), multiple_row=True))).place(x=175, y=100, width=150)
    y_ang_vel = Button(newwin, text="Get y angular velocity", fg="red", command=lambda: print_list(get_y_ang_vel(data, command(entry1), multiple_row=True))).place(x=175, y=125, width=150)
    z_ang_vel = Button(newwin, text="Get z angular velocity", fg="red", command=lambda: print_list(get_z_ang_vel(data, command(entry1), multiple_row=True))).place(x=175, y=150, width=150)


def imu_row_section():
    """
    GUI window for reading IMU data of the row section

    Return:

        A textbox of the parameter data user wanted to read.
    """
    newwin = Toplevel(window)
    newwin.geometry("500x600")

    global t
    t = Text(newwin)
    t.place(x=75, y=250, height=300, width=200)

    Label(newwin, text="Enter Rows").place(x=0, y=0)
    entry1 = Entry(newwin)
    entry1.place(x=0, y=25, width=50)

    imu_time = Button(newwin, text="Get IMU Timestamp", fg="red", command=lambda: print_list(get_IMU_time(data, command(entry1), row_section=True))).place(x=0, y=50, width=150)
    accel_time = Button(newwin, text="Get Accelerometer Timestamp", fg="red", command=lambda: print_list(get_accel_time(data, command(entry1), row_section=True))).place(x=0, y=75, width=150)
    gyro_time = Button(newwin, text="Get Gyroscope Timestamp", fg="red", command=lambda: print_list(get_gyro_time(data, command(entry1), row_section=True))).place(x=0, y=100, width=150)
    x_accel = Button(newwin, text="Get x acceleration", fg="red", command=lambda: print_list(get_x_accel(data, command(entry1), row_section=True))).place(x=0, y=125, width=150)
    y_accel = Button(newwin, text="Get y acceleration", fg="red", command=lambda: print_list(get_y_accel(data, command(entry1), row_section=True))).place(x=175, y=50, width=150)
    z_accel = Button(newwin, text="Get z acceleration", fg="red", command=lambda: print_list(get_z_accel(data, command(entry1), row_section=True))).place(x=175, y=75, width=150)
    x_ang_vel = Button(newwin, text="Get x angular velocity", fg="red", command=lambda: print_list(get_x_ang_vel(data, command(entry1), row_section=True))).place(x=175, y=100, width=150)
    y_ang_vel = Button(newwin, text="Get y angular velocity", fg="red", command=lambda: print_list(get_y_ang_vel(data, command(entry1), row_section=True))).place(x=175, y=125, width=150)
    z_ang_vel = Button(newwin, text="Get z angular velocity", fg="red", command=lambda: print_list(get_z_ang_vel(data, command(entry1), row_section=True))).place(x=175, y=150, width=150)


def xyz_calc():
    """
    GUI window for reading the xyz coordinates gotten from lines of the lidar csv file.

    Return:

        A textbox of the xyz coordinates from rows of the lidar csv file.
    """
    newwin = Toplevel(window)
    newwin.geometry("500x600")

    global t
    t = Text(newwin)
    t.place(x=75, y=250, height=300, width=200)

    row_num = IntVar(newwin)
    row_choice = ['Choose Row']
    for i in range(0, data.shape[0]):
        row_choice.append(i)

    row_num.set('Choose Row')
    popupMenu = OptionMenu(newwin, row_num, *row_choice).place(x=0, y=0, width=175)

    azimuth_block_num = IntVar(newwin)
    azimuth_choice = ['Choose Azimuth Block']
    for i in range(0, 16):
        azimuth_choice.append(i)

    azimuth_block_num.set('Choose Azimuth Block')
    popupMenu = OptionMenu(newwin, azimuth_block_num, *azimuth_choice).place(x=175, y=0, width=175)

    channel_num = IntVar(newwin)
    channel_choice = ['Choose Channel']
    for i in range(0, 64):
        channel_choice.append(i)

    channel_num.set('Choose Channel')
    popupMenu = OptionMenu(newwin, channel_num, *channel_choice).place(x=325, y=0, width=175)

    xyz = Button(newwin, text="Get lidar XYZ coordinates", fg='red', command=lambda: print_list(get_xyz(data, row_num.get(), azimuth_block_num.get(), channel_num.get()))).place(x=175, y=100)


def open_x4_bin():
    """
    Opens the Novelda X4 binary file for reading.
    """
    global X4_file
    X4_file = fd.askopenfilename(filetypes=(("binary file", "*.dat"), ("All files", "*.*")))


def open_x4_csv():
    """
    Opens the Novelda X4 csv file for use in threshold functions.
    """
    global X4_csv
    X4_csv = fd.askopenfilename(filetypes=(("csv file", "*.csv"), ("All files", "*.*")))


def convert_x4():
    """
    GUI window with button to allow conversion of X4 binary file to csv file.

    Return:

        Readable X4 csv file.
    """
    newwin = Toplevel(window)
    newwin.geometry("200x200")

    Label(newwin, text="Enter desired csv file name").place(x=0, y=0)
    entry1 = Entry(newwin)
    entry1.place(x=0, y=25, width=100)

    convert = Button(newwin, text="Convert binary file to csv file", command=lambda: iq_data(X4_file, entry1.get())).place(x=0, y=50)


def x4_threshold():
    """
    GUI window to run X4 threshold functions

    Returns:

        Radar threshold, noise and range estimates
    """
    newwin = Toplevel(window)
    newwin.geometry("800x600")

    global t
    t = Text(newwin)
    t.place(x=75, y=250, height=300, width=200)

    Label(newwin, text="Enter an estimated threshold").place(x=0, y=0)
    entry1 = Entry(newwin)
    entry1.place(x=0, y=25, width=100)

    range_bin = Button(newwin, text="Get range bin(s) above threshold", command=lambda: print_list(range_finder(X4_csv, float(entry1.get())))).place(x=0, y=50)
    noise = Button(newwin, text="Get noise estimate", command=lambda: print_list(noise_power_estimate(X4_csv, float(entry1.get())))).place(x=100, y=50)
    distance = Button(newwin, text="Get target distance", command=lambda: print_list(distance_finder(X4_csv, float(entry1.get())))).place(x=200, y=50)
    plot = Button(newwin, text="Plot data for range vs. amplitude", command=lambda: plot_data(X4_csv)).place(x=300, y=50)


def open_TSW_bin():
    """
    Opens the TSW binary file.
    """
    global TSW_file
    TSW_file = fd.askopenfilename(filetypes=(("binary file", "*.bin"), ("All files", "*.*")))


def convert_TSW():
    """
    Converts the TSW binary file to a readable csv file.
    """
    newwin = Toplevel(window)
    newwin.geometry("200x200")

    Label(newwin, text="Enter desired csv file name").place(x=0, y=0)
    entry1 = Entry(newwin)
    entry1.place(x=0, y=25, width=100)

    convert = Button(newwin, text="Convert binary file to csv file", command=lambda: readTSWdata(TSW_file, entry1.get())).place(x=0, y=50)


def instruction():
    """
    A set of instructions on how to use the program
    """
    newwin = Toplevel(window)
    newwin.geometry("1750x750")

    instruction_img = ImageTk.PhotoImage(Image.open("Instructions.png"))
    instruction_panel = Label(newwin, image=instruction_img)
    instruction_panel.photo = instruction_img

    # Placing images
    instruction_panel.place(x=0, y=0)


""" Options for the file menu """
file_menu.add_command(label="Open", command=openfile)
file_menu.add_command(label="Instructions", command=instruction)
file_menu.add_command(label="Quit", command=quit)

""" Options for the extract menu """
lidar_extract_menu.add_command(label="Single row", command=lidar_single_row)
lidar_extract_menu.add_command(label="Multiple row", command=lidar_multiple_row)
lidar_extract_menu.add_command(label="Row section", command=lidar_row_section)

IMU_extract_menu.add_command(label="Single row", command=imu_single_row)
IMU_extract_menu.add_command(label="Multiple row", command=imu_multiple_row)
IMU_extract_menu.add_command(label="Row section", command=imu_row_section)

xyz_cartesian.add_command(label="Get point", command=xyz_calc)

X4.add_command(label="Open binary file", command=open_x4_bin)
X4.add_command(label="Open converted csv file", command=open_x4_csv)
X4.add_command(label="Convert binary file", command=convert_x4)
X4.add_command(label="Threshold functions", command=x4_threshold)

TSW.add_command(label="Open binary file", command=open_TSW_bin)
TSW.add_command(label="Convert binary file", command=convert_TSW)

window.geometry("700x200")
window.config(menu=menubar) 
window.mainloop()


